@@ -11,8 +11,8 @@ import time
 import pytesseract
 import yaml
 import json
-import difflib
-import unidecode
+from difflib import SequenceMatcher
+from unidecode import unidecode_expect_ascii
 import PIL
 
 # print("Hello sheet music")
@@ -128,17 +128,17 @@ class Detection:
 	def bot(self): return self.__top + self.__height
 
 
+class SubstringSequenceMatcher(SequenceMatcher):
+	def is_rougly_substring(self, substring, fullstring, ratio=0.95):
+		substring = unidecode_expect_ascii(substring.lower())
+		fullstring = unidecode_expect_ascii(fullstring.lower())
+		self.set_seq1(substring)
+		for i in range(max(1, len(fullstring) - len(substring) + 1)):
+			self.set_seq2(fullstring[i : i + len(substring)])
+			if self.quick_ratio() > ratio:
+				return True
+		return False
 
-def isSimilarEnough(detectedText, keyword):
-	return difflib.SequenceMatcher(None, unidecode.unidecode_expect_ascii(detectedText.lower()),
-		unidecode.unidecode_expect_ascii(keyword.lower())).ratio() > 0.9
-		# or \
-	    #        difflib.SequenceMatcher(None, detectedText.lower()+"s", keyword.lower()).ratio() > 0.9 or \
-	    #        difflib.SequenceMatcher(None, detectedText.lower()+"es", keyword.lower()).ratio() > 0.9 or \
-	    #        difflib.SequenceMatcher(None, detectedText.lower()+"r", keyword.lower()).ratio() > 0.9 or \
-	    #        difflib.SequenceMatcher(None, detectedText.lower()+"er", keyword.lower()).ratio() > 0.9 or \
-	    #        difflib.SequenceMatcher(None, detectedText.lower()+"as", keyword.lower()).ratio() > 0.9
-	return detectedText.lower() == keyword.lower()
 
 def predictParts(detectionData, instruments, imageWidth, imageHeight):
 	# return partNames, instrumentses
@@ -153,6 +153,7 @@ def predictParts(detectionData, instruments, imageWidth, imageHeight):
 	# Secondly, gather a list of all matches between detected texts and instruments
 	matches = []
 	exceptionMatches = []
+	sequence_matcher = SubstringSequenceMatcher()
 	for instrument in instruments:
 		for j in range(len(instruments[instrument]["include"])):
 			keyword = instruments[instrument]["include"][j]
@@ -170,7 +171,7 @@ def predictParts(detectionData, instruments, imageWidth, imageHeight):
 					for l in range(len(detectedWords)):
 						detectedWords[l] = detectedWords[l].text()
 					detectedText = " ".join(detectedWords)
-					if isSimilarEnough(detectedText, keyword):
+					if sequence_matcher.is_rougly_substring(keyword, detectedText):
 						matches.append({"i": i, "instrument": instrument, "keyword": keyword})
 
 		for j in range(len(instruments[instrument]["exceptions"])):
@@ -189,7 +190,7 @@ def predictParts(detectionData, instruments, imageWidth, imageHeight):
 					for k in range(len(detectedWords)):
 						detectedWords[k] = detectedWords[k].text()
 					detectedText = " ".join(detectedWords)
-					if isSimilarEnough(detectedText, keyword):
+					if sequence_matcher.is_rougly_substring(keyword, detectedText):
 						exceptionMatches.append({"i": i, "instrument": instrument, "keyword": keyword})
 
 	# Lastly, predict how many, what names, and for what instruments the parts are
@@ -207,7 +208,7 @@ def predictParts(detectionData, instruments, imageWidth, imageHeight):
 				blocksWithMatches.add(detections[match["i"]].block_num())
 			
 		nrOfBlocksWithMatches = len(blocksWithMatches)
-		if nrOfBlocksWithMatches <= 2:
+		if nrOfBlocksWithMatches <= 3:
 			partNames = []
 			instrumentses = []
 			for blockNr in blocksWithMatches:
